import { createContext, useContext, useMemo, PropsWithChildren } from "react";
import { ThemeProvider as StyledThemeProvider } from "styled-components";

import { ThemeMode, lightTheme, darkTheme } from "@/app/themes";
import useLocalStorage from "@/hooks/local_storage";

const ThemeContext = createContext({
  theme: "light",
  toggleTheme: () => { },
} as {
  theme: ThemeMode,
  toggleTheme: () => void
});

export const ThemeProvider = ({ children }: PropsWithChildren) => {
  const [theme, setTheme] = useLocalStorage<ThemeMode>("theme", "light");

  const toggleTheme = () => {
    setTheme(previous => previous === "light" ? "dark" : "light");
  };

  const value = useMemo(() => ({ theme, toggleTheme }), [theme]);

  return (
    <ThemeContext.Provider value={value}>
      <StyledThemeProvider theme={theme === "light" ? lightTheme : darkTheme}>
        {children}
      </StyledThemeProvider>
    </ThemeContext.Provider>
  );
};

export const useTheme = () => {
  const context = useContext(ThemeContext);

  if (!context) {
    throw new Error("useTheme must be used within a ThemeProvider");
  }

  return context;
};